{-----------------------------------------------------------------------------------
Diese Unit ist Teil des Programms USBoot.

Das Programm USBoot ist ein einfaches Programm zum Laden eines HEX-Files
in einen PIC18Fxxxx mit USB-Schnittstelle und USB-Bootloader.
Es bentigt den Bootloader von:
http://sprut.de/electronic/pic/8bit/18f/programm/usbbootloader/usbboot.htm#bootloader
oder den Bootloader 1.0 aus dem USBurn-Paket:
http://sprut.de/electronic/soft/usburn/usburn.htm#download

Nach nderung des vid_pid-Wertes in usbdll.pas kann auch der Microchip USB-Bootloader verwendet werden:
http://www.microchip.com/stellent/idcplg?IdcService=SS_GET_PAGE&nodeId=2124&param=en022627


Copyright (C) 2006  Joerg Bredendiek
 Homepage: http:www.sprut.de
 E-Mail:   sprut@sprut.de

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


Auf Deutsch:
Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen
der GNU General Public License, wie von der Free Software Foundation verffentlicht,
weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
(nach Ihrer Option) jeder spteren Version.

Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen
sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE
oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der
GNU General Public License.

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem Programm erhalten haben.
Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

--------------------------------------------------------------------------------------}

unit brennusbuni;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ToolWin, Clipbrd, syncobjs,
  Buttons, Gauges, ImgList, math, shellapi;
  
const




  ERASE		  = $70;
  REMOVECP	  = $71;
  RESET_DEVICE    = $FF;

  
  sVDD_on         = $01;
  sVDD_off        = $10;
  sVPP_on         = $02;
  sVPP_off        = $20;
  sPGD_on         = $04;
  sPGD_off        = $40;
  sPGC_on         = $08;
  sPGC_off        = $80;
  sActive         = 1;
  sPassive        = 0;
      

var
  iniPfad        : string = 'c:\';  //pfad fr das ini-File
  BrennerTyp     : integer = 0;


function SmallFonts : boolean;     //kompensiere large fonts
procedure SystemInfo(progname : string);
procedure alarm(s:string);         //Fehler ausgeben
procedure info(s:string);          //info ausgeben wenn blabla
procedure addinfo(s:string);       //info anhngen wenn blabla
function USBinfo:boolean;
procedure GrosseFontsKompensieren(Win:Twincontrol);
procedure Sende_Empfange(N,M :byte);
procedure Sende_Zwei_Byte(c,b:byte);
procedure Sende_Drei_Byte(a,b,c:byte);
procedure NBytesLEsen(code,n:byte);


implementation


uses
   usbdll, usbboot;


var
  k   : integer;
  lf  : boolean;  //large fonts


//N Bytes senden und M Bytes empfangen
//timeout ist jeweils 100 ms
procedure Sende_Empfange(N,M :byte);
var
   selection   : DWORD;
   RecvLength  : DWORD;
   fehler      : integer;
begin
  if not (BrennerDa or BootloaderDa) then exit;
  //instance ist warscheinlich 0, kann aber auch einen anderen Wert haben
  selection:=0;

  //pipes anfordern
  //fr die instance=0
  //es kann aber passieren, dass instance<>0 !!
  //da ist noch nacharbeit ntig
  myOutPipe:= _MPUSBOpen(selection,vid_pid,out_pipe,MP_WRITE,0);
  myInPipe := _MPUSBOpen(selection,vid_pid,out_pipe,MP_READ,0);
  //habe ich pipes bekommen?
  if ((myOutPipe = INVALID_HANDLE_VALUE) or (myInPipe = INVALID_HANDLE_VALUE)) then
  begin
    info('USB Error, no pipes');
    exit;
  end;

  RecvLength:=M;
  fehler:=SendReceivePacket(send_buf,N,receive_buf,RecvLength,100,1000);
  if Fehler<>1 then  info('USB Error :'+inttostr(fehler));

  _MPUSBClose(myOutPipe);
  _MPUSBClose(myInPipe);
  myInPipe:= INVALID_HANDLE_VALUE;
  myOutPipe:=INVALID_HANDLE_VALUE;
end; // sende_empfange


//2 Bytes senden
procedure Sende_Zwei_Byte(c,b:byte);
begin
//  if not BrennerDa then exit; // das erledigt Sende_Empfange
  send_buf[0]:=c;
  send_buf[1]:=b;
  Sende_Empfange(2,1);
end;


//3 Bytes senden
procedure Sende_Drei_Byte(a,b,c:byte);
begin
//  if not BrennerDa then exit; // das erledigt Sende_Empfange
  send_buf[0]:=a;
  send_buf[1]:=b;
  send_buf[2]:=c;
  Sende_Empfange(3,1);
end;


procedure NBytesLEsen(code,n:byte);
begin
  send_buf[0]:=code;
  Sende_Empfange(1,n+1);
end;


//Fehler ausgeben
procedure alarm(s:string);
begin
  bootloader.memo18.lines.add('## '+s);
end;


//info ausgeben wenn blabla
procedure info(s:string);
begin
  if blabla then bootloader.memo18.lines.add(s);
end;


//info anhngen wenn blabla
procedure addinfo(s:string);
begin
  if blabla then bootloader.memo18.lines[bootloader.memo18.lines.count-1]:=
                   bootloader.memo18.lines[bootloader.memo18.lines.count-1]+s;
end;


//kompensiere large fonts
function SmallFonts : boolean;
var dc : hdc;
begin
  dc:=GetDC(0);
  Result:=(GetDeviceCaps(dc, LOGPIXELSX) =96);
  ReleaseDC(0, DC);
end;


procedure SystemInfo(progname : string);
var VersionInfo : TOSVersionInfo;
begin
  info(progname+Version);
  //Betriebssystem
  // 0: Win 3.1
  // 1: Win9x
  // 2: Win2k
  VersionInfo.dwOSVersionInfoSize:=SizeOf(VersionInfo);
  GetVersionEx(VersionInfo);
  case  VersionInfo.dwPlatformId of
    0: info('-- Windows 3.1');
    1: info('-- Windows 95/98/me');
    2: info('-- Windows NT/2000/XP');
    else  info('OS-Type: '+inttostr(VersionInfo.dwPlatformId));
  end;
  info('-- Version '+inttostr(VersionInfo.dwMajorVersion)+'.'+
        inttostr(VersionInfo.dwMinorVersion)+' Build '+inttostr(VersionInfo.dwBuildNumber));
  info('-- '+VersionInfo.szCSDVersion);
end;



// infos ber usb treiber und brenner
// liefert false, wenn kein Brenner da ist
function USBinfo:boolean;
var zahl : integer;
begin
  result:=false;
  info('-- USB-DLL-Version : '+inttohex(_MPUSBGetDLLVersion,8));
  zahl:=_MPUSBGetDeviceCount(vid_pid);
  if zahl<1 then begin
    alarm(' NO device detected!');
    exit;
  end else begin
    info('-- '+inttostr(zahl)+' device(s) detected');
    result:=true;
  end;
end;


procedure GrosseFontsKompensieren(Win:Twincontrol);
var DC  : HDC;
begin
  lf:=false;
  if not SmallFonts then begin
    lf:=true;
    DC:=GetDC(0);
    Win.ScaleBy(96,GetDeviceCaps(DC, LOGPIXELSX));
    ReleaseDC(0, DC);
    win.Refresh;
  end;
end;



// ENDE HARDWARENAHE ROUTINEN
//##############################################################################


initialization

  //Pfad fr ini-Datei festlegen (=Programmverzeichnis)
  iniPfad:=getcurrentdir+'\';

finalization


end.
